Documentation
¶
Overview ¶
Package abort implements the stackit abort command for canceling in-progress operations.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type Handler ¶
type Handler interface {
// PromptConfirmAbort prompts user to confirm aborting the current operation
// Returns true to proceed with abort, false to cancel
PromptConfirmAbort() (bool, error)
// Cleanup restores terminal state (may be no-op)
Cleanup()
// IsInteractive returns true if the handler supports interactive prompts
IsInteractive() bool
}
Handler receives events from abort action
type NullHandler ¶
NullHandler is a no-op handler for when nil is passed
func (*NullHandler) PromptConfirmAbort ¶
func (h *NullHandler) PromptConfirmAbort() (bool, error)
PromptConfirmAbort implements Handler. Returns false (cancel) for null handler.
Click to show internal directories.
Click to hide internal directories.