Documentation
¶
Overview ¶
Package doctor provides diagnostic functionality for checking stackit environment and repository health.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type CheckStatus ¶
type CheckStatus string
CheckStatus represents the result of a diagnostic check
const ( CheckPassed CheckStatus = "passed" CheckWarning CheckStatus = "warning" CheckError CheckStatus = "error" )
Check status constants
type Handler ¶
type Handler interface {
// Start is called at the beginning of doctor
Start(fix bool)
// OnCategory is called when starting a new category of checks
OnCategory(category Category)
// OnCheck is called for each individual check result
OnCheck(name string, status CheckStatus, message string)
// Complete is called when doctor finishes
Complete(passed, warnings, errors int)
// Cleanup restores terminal state (may be no-op)
Cleanup()
// IsInteractive returns true if the handler supports interactive prompts
IsInteractive() bool
}
Handler receives events from doctor action
type NullHandler ¶
NullHandler is a no-op handler for when nil is passed. It embeds handler.NullBase for Cleanup() and IsInteractive().
func (*NullHandler) Complete ¶
func (h *NullHandler) Complete(int, int, int)
Complete implements Handler.
func (*NullHandler) OnCategory ¶
func (h *NullHandler) OnCategory(Category)
OnCategory implements Handler.
func (*NullHandler) OnCheck ¶
func (h *NullHandler) OnCheck(string, CheckStatus, string)
OnCheck implements Handler.
Click to show internal directories.
Click to hide internal directories.