timeutil

package
v0.17.14 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: May 11, 2026 License: MIT Imports: 2 Imported by: 0

Documentation

Overview

Package timeutil provides utilities for time formatting and manipulation.

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func FormatTimeAgo

func FormatTimeAgo(timestamp time.Time) string

FormatTimeAgo formats a timestamp as a relative time string

Types

This section is empty.

Jump to