Documentation
¶
Overview ¶
Package init provides functionality for initializing Stackit in a Git repository.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func FindCommonlyNamedTrunk ¶
FindCommonlyNamedTrunk checks for common trunk branch names Returns the branch name if exactly one is found, empty string otherwise
Types ¶
type Handler ¶
type Handler interface {
// SelectTrunk prompts the user to select the trunk branch
SelectTrunk(ctx context.Context, branchNames []string, inferredTrunk string) (string, error)
// OnSuccess is called when initialization finishes
OnSuccess(trunkName string, wasInitialized bool, isReset bool)
}
Handler abstracts interaction for the init action
Click to show internal directories.
Click to hide internal directories.