Documentation
¶
Overview ¶
Package sync provides a TUI component for displaying sync progress.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
This section is empty.
Types ¶
type CompleteMsg ¶
type CompleteMsg struct {
Summary string
}
CompleteMsg indicates sync is complete
type Model ¶
type Model struct {
core.BaseModel // Embedded for ReadySignaler interface
CurrentPhase Phase
CurrentDetail string // Current operation being performed
TotalOps int
CompletedOps int
Progress progress.Model
Summary string
// contains filtered or unexported fields
}
Model is the bubbletea model for sync progress. It embeds core.BaseModel for standard lifecycle handling. Uses tea.Printf to print completed items above the active UI.
type PhaseCompleteMsg ¶
type PhaseCompleteMsg struct {
Phase Phase
}
PhaseCompleteMsg indicates a phase has completed
type PhaseDetailMsg ¶
type PhaseDetailMsg struct {
Phase Phase
Message string
IsWarn bool // If true, shows ⚠ instead of ✓
}
PhaseDetailMsg adds a detail line to a phase (printed above TUI)
type PhaseStartMsg ¶
type PhaseStartMsg struct {
Phase Phase
Message string // Phase header message (e.g., "📥 Pulling from remote...")
}
PhaseStartMsg indicates a phase has started
type ProgressTickMsg ¶
ProgressTickMsg updates the progress bar
Click to show internal directories.
Click to hide internal directories.